package cn.com.ty.lift.business.annualinspection.controller;


import cn.com.ty.lift.business.annualinspection.dto.InspectionRequest;
import cn.com.ty.lift.business.annualinspection.dto.InspectionResponse;
import cn.com.ty.lift.business.annualinspection.entity.AnnualInspection;
import cn.com.ty.lift.business.annualinspection.service.AnnualInspectionService;
import cn.com.ty.lift.business.common.CountPage;
import cn.com.ty.lift.business.common.Judge;
import cn.com.ty.lift.business.common.Values;
import cn.com.ty.lift.business.framework.conf.SystemConfiguration;
import cn.com.ty.lift.common.utils.AliyunOSSUtil;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.time.LocalDate;
import java.time.LocalDateTime;

/**
 * <p>
 * 年检记录 前端控制器
 * </p>
 *
 * @author wcz
 * @since 2019-12-10
 */
@Slf4j
@AllArgsConstructor
@RestController
@RequestMapping("/annualinspection/annual-inspection")
public class AnnualInspectionController {

    private AnnualInspectionService annualInspectionService;

    private SystemConfiguration systemConfiguration;

    private AliyunOSSUtil aliyunOSSUtil;

    /**
     * @description 根据id查询
     * @date 2019/12/10 10:19
     * @param
     * @return
     */
    @PostMapping("findOne")
    public RestResponse findOne(@RequestBody InspectionRequest request){
        Long id = request.getId();
        Judge.id(id);
        InspectionResponse entity = annualInspectionService.selectByIdWithInfo(request);
        return Judge.result(entity);
    }

    /**
     * @description 根据条件查询年检记录
     * @date 2019/12/11 17:15
     * @param
     * @return
     */
    @PostMapping("page")
    public RestResponse page(@RequestBody InspectionRequest request){
        String info = request.getInfo();
        if(StrUtil.isNotBlank(info)){
            request.setInfo(StrUtil.format("%{}%",info));
        }

        IPage<InspectionResponse> pages = annualInspectionService.pageByCondition(request);
        Judge.notNull(pages);
        long count = annualInspectionService.countByCondition(request);
        CountPage<InspectionResponse> countPage = new CountPage<>(count, pages);
        return RestResponse.success(countPage);
    }

    /**
     * @description 根据条件查询年检记录
     * @date 2019/12/11 17:15
     * @param
     * @return
     */
    @PostMapping("pageWorker")
    public RestResponse pageWorker(@RequestBody InspectionRequest request){
        Long workerId = request.getWorkerId();
        Judge.id(workerId);
        IPage<InspectionResponse> pages = annualInspectionService.pageByCondition(request);
        return Judge.result(pages);
    }
    /**
     * @description 修改年检信息
     *
     * status 状态（0 待完成；1 已完成；2 超期）
     *
     * stepStatus  :年检阶段状态
     *
     * 未开始0，第一阶段 1-2，第二阶段 3，第三阶段 4-7，第四阶段 8-10
     *
        1  维保工确认年检计划
        2  维保工上传自检报告,选择自检时间 或者 文员设置 是否检查限速器和荷载年检设置，
            文员也可以上传自检报告或者确认自检报告和自检时间，自检时间以最后设置的为准
        --------------------------------
        3  企业文员确认现场检验时间
            录入人员信息，政府质检，企业质检，联系电话
        --------------------------------
        4	维保工上传检验结果
        5	企业文员确认检验结果-合格 --> 不能算年检超期
        6	企业文员确认检验结果-整改 --> 回到第二阶段
        7	企业文员确认检验结果-不合格 --> 可以算年检超期
        --------------------------------
        8	维保工上传年检报告和合格证
        9	企业文员确认年检报告和合格证-合格
        10	企业文员确认年检报告和合格证-不合格 -->停用电梯
     * @date 2019/12/10 10:04
     * @param
     * @return
     */
    @PostMapping("modify")
    public RestResponse modify(@RequestBody AnnualInspection entity){
        Long id = entity.getId();
        Judge.id(id);
        AnnualInspection old = annualInspectionService.getById(id);
        Judge.notNull(old);
        //旧状态
        Integer oldStepStatus = old.getStepStatus();
        //新状态
        Integer stepStatus = entity.getStepStatus();

        //0 --> 1 维保工确认年检计划
        Judge.notTrue(Values.aiStatusNotStart(oldStepStatus) && !Values.aiSameNext(oldStepStatus,stepStatus),"请执行[确认年检计划]操作");
        /**
         * 1 --> 2
         * 2  维保工上传自检报告,选择自检时间 或者 文员设置 是否检查限速器和荷载年检设置，
         *             文员也可以上传自检报告或者确认自检报告和自检时间，自检时间以最后设置的为准
         */
        Judge.notTrue(Values.aiStatusComfirm(oldStepStatus) && !Values.aiNextStatus(oldStepStatus,stepStatus),"请执行[上传自检报告]操作");
        /**
         * 2  维保工上传自检报告,选择自检时间 或者 文员设置 是否检查限速器和荷载年检设置，
         *             文员也可以上传自检报告或者确认自检报告和自检时间，自检时间以最后设置的为准
         */
        if(Values.aiStatusSelfcheck(oldStepStatus)){
            // 修改自检信息
            if(Values.aiSameStatus(oldStepStatus,stepStatus)){
                Judge.notNull(entity.getSelfcheckReportImg(),"请执行[上传自检报告]操作");
                Judge.notNull(entity.getSelfcheckDate(),"请确认自检时间");
                //流转到下一个状态 2 --> 3
            }else if(Values.aiStatusCheckComfirm(stepStatus)){
                //3  企业文员确认现场检验时间，录入人员信息，政府质检，企业质检，联系电话
                Judge.notNull(entity.getCheckDate(),"请设置现场检验时间");
                Judge.notNull(entity.getInspector(),"请设置政府质检人员");
                Judge.notNull(entity.getOperator(),"请设置公司质检人员");
                Judge.notNull(entity.getTelephone(),"请设置人员联系电话");
            }else {
                return RestResponse.fail("请执行[确认现场检验时间]操作");
            }
        }
        // 3 --> 4  维保工上传检验结果
        Judge.notTrue(Values.aiStatusCheckComfirm(oldStepStatus) && !Values.aiNextStatus(oldStepStatus,stepStatus),"请执行[上传检验结果]操作");
        //4 检查上传检验结果
        if(Values.aiStatusCheckUpload(oldStepStatus)){
            //修改检验结果
            if(Values.aiSameStatus(oldStepStatus,stepStatus)){
                Judge.notNull(entity.getCheckResultImg(),"请执行[上传检验结果]操作");
                Judge.notNull(entity.getCheckDate(),"请执行[确认检验时间]操作");
                //流转到下一个状态
            }else{
            /**
             *  5	企业文员确认检验结果-合格
             *  6	企业文员确认检验结果-整改
             *  7	企业文员确认检验结果-不合格
             */
                Judge.notTrue(!Values.aiInStatus(Values.AI_STATUS_CHECK_QUALIFIED,stepStatus,Values.AI_STATUS_CHECK_UNQUALIFIED),"请执行[确认检验结果]操作");
            }
        }
        /**
         * 6 检验结果整改，回到阶段2
         * (3  企业文员确认现场检验时间 录入人员信息，政府质检，企业质检，联系电话)
         */
        if(Values.aiStatusCheckRectification(stepStatus)){
            entity.setStepStatus(Values.AI_STATUS_CHECK_COMFIRM);
        }
        /**
         * 5	企业文员确认检验结果-合格 --> 8	维保工上传年检报告和合格证
         * 7	企业文员确认检验结果-不合格-->8	维保工上传年检报告和合格证
         */
        if(Values.aiStatusCheckQualified(oldStepStatus) || Values.aiStatusCheckUnqualified(oldStepStatus)){
            if(Values.aiStatusCertificateReportUpload(stepStatus)){
                Judge.notTrue(StrUtil.hasEmpty(entity.getCertificateImgUrl(),entity.getAnnualInspectionImg()),"请执行[上传年检报告和合格证]操作");
            }else {
                return RestResponse.fail("年检记录状态有误");
            }
        }
        /**
         * 8	维保工上传年检报告和合格证--> 9  企业文员确认年检报告和合格证 -合格
         * 8	维保工上传年检报告和合格证--> 10  企业文员确认年检报告和合格证 -不合格
         */
        if(Values.aiStatusCertificateReportUpload(oldStepStatus)){
            //修改年检报告和合格证
            if(stepStatus == oldStepStatus){
                Judge.notTrue(StrUtil.hasEmpty(entity.getCertificateImgUrl(),entity.getAnnualInspectionImg()),"请执行[上传年检报告和合格证]操作");
            }else if(Values.aiStatusCertificateReportQualified(stepStatus)){
                //合格：文员确认年检报告并设置下次年检时间
                Judge.notNull(entity.getNextInspectionTime(),"请设置下次年检时间");
                // 设置年检完成 status = 1
                entity.setFinishTime(LocalDateTime.now());
                entity.setStatus(1);
            }else if(Values.aiStatusCertificateReportUnqualified(stepStatus)){
                // 设置年检完成 status = 1
                entity.setFinishTime(LocalDateTime.now());
                entity.setStatus(1);
            }else{
                return RestResponse.fail("年检记录状态有误");
            }
        }

        int stepName = Values.aiStatus2StepName(entity.getStepStatus());
        Judge.notTrue(stepName == -1,"年检状态有误,状态码: " + stepStatus);
        entity.setStepName(stepName);

        boolean result = annualInspectionService.finish(entity);
        return Judge.result(result);
    }

    @GetMapping("show")
    public BufferedImage show(@RequestParam("filename") String filename) throws IOException {
        if(StrUtil.isEmpty(filename)){
            return null;
        }
        String fileurl = aliyunOSSUtil.getObjectUrl(systemConfiguration.bucketName,filename);
        log.info("fileurl: " + fileurl);
        File file = new File("download/" + filename);
        File down = aliyunOSSUtil.getFile(systemConfiguration.bucketName,filename,file);
        return null == down ? null : ImageIO.read(new FileInputStream(file));
    }
    /**
     * 其他角色获取自己关联的项目的年检任务
     * @param request
     * @return
     */
    @PostMapping("pageByUser")
    public RestResponse pageByUser(@RequestBody InspectionRequest request){
        Long userId = request.getUserId();
        Judge.id(userId);
        IPage<InspectionResponse> pages = annualInspectionService.pageByUser(request);
        return Judge.result(pages);
    }

    /**
     * 确认年检计划，传递年检id，后台修改年检状态，停止信息推送
     * @param request
     * @return
     */
    @PostMapping("confirm")
    public RestResponse confirm(@RequestBody InspectionRequest request){
        Long id = request.getId();
        Judge.id(id);
        AnnualInspection entity = annualInspectionService.getById(id);
        Judge.notNull(entity);
        entity.setStepStatus(Values.AI_STATUS_COMFIRM);
        entity.setStepName(Values.AI_STEP_ONE);
        boolean result = annualInspectionService.updateById(entity);
        return Judge.result(result);
    }

    /**
     * 上传自检报告，传递年检id，传递自检报告图片路径
     * @return
     */
    @PostMapping("selfcheck")
    public RestResponse selfcheck(@RequestBody InspectionRequest request){
        Long id = request.getId();
        Judge.id(id);
        LocalDate selfcheckDate = request.getSelfcheckDate();
        String selfcheckReportImg = request.getSelfcheckReportImg();
        Judge.notNull(selfcheckDate,"缺少自检时间");
        Judge.notNull(selfcheckReportImg,"缺少自检图片");
        AnnualInspection entity = annualInspectionService.getById(id);
        Judge.notNull(entity);
        entity.setSelfcheckDate(selfcheckDate);
        entity.setSelfcheckReportImg(selfcheckReportImg);
        entity.setStepStatus(Values.AI_STATUS_SELFCHECK);
        entity.setStepName(Values.AI_STEP_ONE);
        boolean result = annualInspectionService.updateById(entity);
        return Judge.result(result);
    }

    /**
     * 上传检验结果，传递年检id，传递检验报告图片路径
     * @param request
     * @return
     */
    @PostMapping("check")
    public RestResponse check(@RequestBody InspectionRequest request){
        Long id = request.getId();
        Judge.id(id);
        LocalDate checkDate = request.getCheckDate();
        String checkResultImg = request.getCheckResultImg();
        Judge.notNull(checkDate,"缺少检验时间");
        Judge.notNull(checkResultImg,"缺少检验图片");
        AnnualInspection entity = annualInspectionService.getById(id);
        Judge.notNull(entity);
        entity.setCheckDate(checkDate);
        entity.setCheckResultImg(checkResultImg);
        entity.setStepStatus(Values.AI_STATUS_CHECK_UPLOAD);
        entity.setStepName(Values.AI_STEP_THREE);
        boolean result = annualInspectionService.updateById(entity);
        return Judge.result(result);
    }

    /**
     * 上传年检报告和合格证，传递年检id，传递年检报告和合格证图片路径
     * @param request
     * @return
     */
    @PostMapping("finish")
    public RestResponse finish(@RequestBody InspectionRequest request){
        Long id = request.getId();
        Judge.id(id);
        String certificateImgUrl = request.getCertificateImgUrl();
        Judge.notNull(certificateImgUrl,"缺少合格证信息");
        String annualInspectionImg = request.getAnnualInspectionImg();
        Judge.notNull(annualInspectionImg,"缺少年检报告信息");
        AnnualInspection entity = annualInspectionService.getById(id);
        Judge.notNull(entity);
        entity.setCertificateImgUrl(certificateImgUrl);
        entity.setAnnualInspectionImg(annualInspectionImg);
        entity.setStepStatus(Values.AI_STATUS_CERTIFICATE_REPORT_UPLOAD);
        entity.setStepName(Values.AI_STEP_FOUR);
        boolean result = annualInspectionService.updateById(entity);
        return Judge.result(result);
    }
}
