package cn.com.ty.lift.enterprise.oa.controller;

import cn.com.ty.lift.enterprise.common.CountPage;
import cn.com.ty.lift.enterprise.common.Judge;
import cn.com.ty.lift.enterprise.oa.dto.AttendanceRequest;
import cn.com.ty.lift.enterprise.oa.dto.AttendanceResponse;
import cn.com.ty.lift.enterprise.oa.entity.Attendance;
import cn.com.ty.lift.enterprise.oa.entity.MaintenanceCompany;
import cn.com.ty.lift.enterprise.oa.service.AttendanceService;
import cn.com.ty.lift.enterprise.oa.service.MaintenanceCompanyService;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;
import java.time.LocalTime;

/**
 * <p>
 * 打卡签到记录 前端控制器
 * </p>
 *
 * @author wcz
 * @since 2019-12-04
 */
@Slf4j
@AllArgsConstructor
@RestController
@RequestMapping("/oa/attendance")
public class AttendanceController {

    private AttendanceService attendanceService;
    private MaintenanceCompanyService maintenanceCompanyService;

    /**
     * @description 根据id查找
     * @date 2019/12/6 9:44
     * @param
     * @return
     */
    @PostMapping("findOne")
    public RestResponse findOne(@RequestBody AttendanceRequest request) {
        Long id = request.getId();
        Judge.id(id);
        Attendance entity = attendanceService.getById(id);
        return Judge.result(entity);
    }

    /**
     * @description 根据条件查询
     * @date 2019/12/6 9:44
     * @param
     * @return
     */
    @PostMapping("list")
    public RestResponse list(@RequestBody AttendanceRequest request){
        String name = request.getUsername();
        if(StrUtil.isNotBlank(name)){
            request.setUsername(StrUtil.format("%{}%",name));
        }

        IPage<AttendanceResponse> pages = attendanceService.pageByCondition(request);
        Judge.notNull(pages);
        long count = attendanceService.count();
        CountPage<AttendanceResponse> countPage = new CountPage<>(count,pages);
        return RestResponse.success(countPage);
    }

    /**
     * @description 设置打卡时间
     * @date 2019/12/20 16:30
     * @param
     * @return
     */
    @PostMapping("setupTime")
    public RestResponse setupTime(@RequestBody AttendanceRequest request){
        LocalTime topTime = request.getTopTime();
        LocalTime downTime = request.getDownTime();
        Judge.notTrue(ObjectUtil.hasEmpty(topTime,downTime),"请设置上午的上下班时间");
        Judge.notTrue(topTime.isAfter(downTime),"上午的上下班时间有误");

        LocalTime topTimePm = request.getTopTimePm();
        LocalTime downTimePm = request.getDownTimePm();
        Judge.notTrue(ObjectUtil.hasEmpty(topTimePm,downTimePm),"请设置下午的上下班时间");
        Judge.notTrue(topTimePm.isAfter(downTimePm),"下午的上下班时间有误");

        LocalTime topTimeOt = request.getTopTimeOt();
        LocalTime downTimeOt = request.getDownTimeOt();
        Judge.notTrue(ObjectUtil.hasEmpty(topTimeOt,downTimeOt),"请设置加班的上下班时间");
        Judge.notTrue(topTimeOt.isAfter(downTimeOt),"加班的上下班时间有误");

        Long mtCompanyId = request.getMtCompanyId();
        Judge.id(mtCompanyId);
        MaintenanceCompany entity = maintenanceCompanyService.getById(mtCompanyId);
        Judge.notNull(entity);
        entity.setTopTime(topTime);
        entity.setDownTime(downTime);
        entity.setTopTimePm(topTimePm);
        entity.setDownTimePm(downTimePm);
        entity.setTopTimeOt(topTimeOt);
        entity.setDownTimeOt(downTimeOt);
        boolean result = maintenanceCompanyService.saveOrUpdate(entity);
        return Judge.result(result);
    }

    /**
     * 打卡
     * 11 上午上班 12 上午下班
     * 21 下午上班 22 下午下班
     * 31 加班上班 32 加班下班
     *
     * @param request
     * @return
     */
    @PostMapping("clock")
    public RestResponse clock(@RequestBody AttendanceRequest request) {
        String coordinate = request.getCoordinate();
        String address = request.getAddress();
        Judge.notTrue(StrUtil.equalsAny(coordinate, address),"缺少地址信息");

        Long mtCompanyId = request.getMtCompanyId();
        Judge.id(mtCompanyId);
        MaintenanceCompany maintenanceCompany = maintenanceCompanyService.getById(mtCompanyId);
        Judge.notNull(maintenanceCompany);
        Attendance attendance = new Attendance();
        attendance.setMtCompanyId(mtCompanyId);
        //fixme:获取当前用户?
        attendance.setUserId(1l);
        attendance.setCoordinate(coordinate);
        attendance.setAddress(address);
        LocalDateTime localDateTime = LocalDateTime.now();
        attendance.setCreateDate(localDateTime);
        LocalTime now = localDateTime.toLocalTime();
        int type = request.getType();
        attendance.setType(type);
        //状态（是否迟到早退，0：否，1：是）
        switch (type) {
            case 11:
                LocalTime topTime = maintenanceCompany.getTopTime();
                Judge.notNull(topTime,"请先设置上午的上班时间");
                //打卡时间比上班时间早-正常，否则迟到
                attendance.setStatus(now.isBefore(topTime) ? 0 : 1);
                attendance.setRemarks("上午上班打卡");
                break;
            case 12:
                LocalTime downTime = maintenanceCompany.getDownTime();
                Judge.notNull(downTime,"请先设置上午的下班时间");
                //打卡时间比下班时间晚-正常，否则早退
                attendance.setStatus(now.isAfter(downTime) ? 0 : 1);
                attendance.setRemarks("上午下班打卡");
                break;
            case 21:
                LocalTime topTimePm = maintenanceCompany.getTopTimePm();
                Judge.notNull(topTimePm,"请先设置下午的上班时间");
                attendance.setStatus(now.isBefore(topTimePm) ? 0 : 1);
                attendance.setRemarks("下午上班打卡");
                break;
            case 22:
                LocalTime downTimePm = maintenanceCompany.getDownTimePm();
                Judge.notNull(downTimePm,"请先设置下午的下班时间");
                attendance.setStatus(now.isAfter(downTimePm) ? 0 : 1);
                attendance.setRemarks("下午下班打卡");
                break;
            case 31:
                LocalTime topTimeOt = maintenanceCompany.getTopTimeOt();
                Judge.notNull(topTimeOt,"请先设置加班的上班时间");
                attendance.setStatus(now.isBefore(topTimeOt) ? 0 : 1);
                attendance.setRemarks("加班上班打卡");
                break;
            case 32:
                LocalTime downTimeOt = maintenanceCompany.getDownTimeOt();
                Judge.notNull(downTimeOt,"请先设置加班的下班时间");
                attendance.setStatus(now.isAfter(downTimeOt) ? 0 : 1);
                attendance.setRemarks("加班下班打卡");
                break;
            default:
                return RestResponse.failParam();
        }
        boolean result = attendanceService.save(attendance);
        return Judge.result(result);
    }

    /**
     * 返回维保公司打卡时间信息
     * @param request
     * @return
     */
    @PostMapping("time")
    public RestResponse time(@RequestBody AttendanceRequest request) {
        Long mtCompanyId = request.getMtCompanyId();
        Judge.id(mtCompanyId);
        MaintenanceCompany entity = maintenanceCompanyService.getById(mtCompanyId);
        return Judge.result(entity);
    }

    /**
     *
     * @param request
     * @return
     */
    @PostMapping("pageByUser")
    public RestResponse pageByUser(@RequestBody AttendanceRequest request){

        Long userId = request.getUserId();
        Judge.id(userId);
        IPage<AttendanceResponse> pages = attendanceService.pageByUser(request);
        return Judge.result(pages);
    }
}
