package cn.com.ty.lift.enterprise.oa.controller;

import cn.com.ty.lift.enterprise.common.CountPage;
import cn.com.ty.lift.enterprise.common.Judge;
import cn.com.ty.lift.enterprise.oa.dto.LiftCertificateRequest;
import cn.com.ty.lift.enterprise.oa.dto.LiftCertificateResponse;
import cn.com.ty.lift.enterprise.oa.entity.LiftCertificate;
import cn.com.ty.lift.enterprise.oa.service.LiftCertificateService;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;


/**
 * <p>
 * 操作证 前端控制器
 * </p>
 *
 * @author wcz
 * @since 2019-12-04
 */
@Slf4j
@AllArgsConstructor
@RestController
@RequestMapping("/oa/liftCertificate")
public class LiftCertificateController {

    private LiftCertificateService liftCertificateService;

    /**
     * @description 根据id查找
     * @date 2019/12/6 9:45
     * @param
     * @return
     */
    @PostMapping("findOne")
    public RestResponse findOne(@RequestBody LiftCertificateRequest request) {
        Long id = request.getId();
        Judge.id(id);
        LiftCertificate entity = liftCertificateService.getById(id);
        return Judge.result(entity);
    }

    /**
     * @description 根据条件查询
     * @date 2019/12/6 9:45
     * @param
     * @return
     */
    @PostMapping("list")
    public RestResponse list(@RequestBody LiftCertificateRequest request) {

        String name = request.getUsername();
        //name 模糊查询
        if (StrUtil.isNotBlank(name)) {
            request.setUsername(StrUtil.format("%{}%",name));
        }
        IPage<LiftCertificateResponse> pages = liftCertificateService.pageByCondition(request);
        Judge.notNull(pages);
        long count = liftCertificateService.countByCondition(request);
        CountPage<LiftCertificateResponse> countPage = new CountPage<>(count, pages);
        return RestResponse.success(countPage);
    }

    /**
     * @description 更新操作证信息
     * @date 2019/12/4 10:32
     * @param
     * @return
     */
    @PostMapping("modify")
    public RestResponse modify(@RequestBody LiftCertificate entity) {
        Long id = entity.getId();
        Judge.id(id);
        Long ownerId = entity.getOwnerId();
        Long mtCompanyId = entity.getMtCompanyId();
        Judge.ids(ownerId,mtCompanyId);
        boolean result = liftCertificateService.updateById(entity);
        return Judge.result(result);
    }

    /**
     * @description 操作证审核
     * @date 2019/12/4 10:32
     * @param
     * @return
     */
    @PostMapping("audit")
    public RestResponse audit(@RequestBody LiftCertificateRequest request) {
        Long id = request.getId();
        Judge.id(id);
        //0：无证，1：待审核，2：审核未通过，3：审核通过，4：超期
        LiftCertificate liftCertificate = liftCertificateService.getById(id);
        Judge.notNull(liftCertificate);
        int status = request.isPass() ? 3 : 2;
        liftCertificate.setStatus(status);
        boolean result = liftCertificateService.saveOrUpdate(liftCertificate);
        return Judge.result(result);
    }

    /**
     * @description 新增操作证
     * @date 2019/12/4 10:31
     * @param
     * @return
     */
    @PostMapping("add")
    public RestResponse add(@RequestBody LiftCertificate entity) {
        Long ownerId = entity.getOwnerId();
        Long mtCompanyId = entity.getMtCompanyId();
        Judge.ids(ownerId,mtCompanyId);
        Judge.notTrue(StrUtil.hasEmpty(entity.getFirstImgUrl(),entity.getSecondImgUrl()),"请上传操作证图片");
        int count = liftCertificateService.countByUserAndMtCompany(ownerId,mtCompanyId);
        Judge.notTrue(count > 0,"操作证已存在");
        boolean result = liftCertificateService.saveOrUpdate(entity);
        return Judge.result(result);

    }
}
